package com.winhc.bigdata.spark.jobs

import java.util.Date

import com.winhc.bigdata.calc.{DimScore, DimScoreV2}
import com.winhc.bigdata.spark.utils.SparkUtils
import org.apache.commons.logging.LogFactory
import org.apache.spark.broadcast.Broadcast
import org.apache.spark.sql.{Row, SparkSession}

import scala.collection.mutable

object CompanyInfoCalculatorV2 {

  private val LOG = LogFactory.getLog(this.getClass)

  def main(args: Array[String]): Unit = {

    if (args.length != 3) {
      println("请配置计算资源: instances, cores, memory .")
      System.exit(-1)
    }

    var config = mutable.Map.empty[String, String]
    val Array(instances, cores, memory) = args;

    println(
      s"""
         |instances : $instances,
         |cores : $cores,
         |memory : $memory
         |""".stripMargin)

    config = mutable.Map("spark.executor.instances" -> instances,
      "spark.executor.cores" -> cores,
      "spark.executor.memory" -> memory
    )

    val spark: SparkSession = SparkUtils.InitEnv(this.getClass.getSimpleName, config)

    import spark.implicits._
    import spark._

    println("company calc start! " + new Date().toString)

    val tableName = "ads_company"
    val resultTable = "ads_company_score_v2"
    val ds = "20200526"

    //所属行业
    val code2Name: Broadcast[Map[String, String]] = spark.sparkContext.broadcast(sql(
      """
        |select category_code,category_str_big
        |from const_company_category_code
      """.stripMargin).collect().map(r => {
      (r.getString(0), r.getString(1))
    }).toMap)

    val df = sql(
      s"""
         |SELECT  id
         |        ,company_name
         |        ,legal_person_name
         |        ,reg_capital
         |        ,reg_capital_num
         |        ,actual_capital
         |        ,category_code
         |        ,company_type
         |        ,company_org_type
         |        ,reg_status
         |        ,cast(estiblish_time as string) estiblish_time
         |        ,cast(from_time as string) from_time
         |        ,cast(to_time as string) to_time
         |        ,reg_location
         |FROM    ${tableName}
         |""".stripMargin).flatMap(r => {
      trans(code2Name, r)
    }).toDF("id", "company_name", "kind", "project", "type", "score", "total", "extraScore")

    // 写 分区表
    df.createOrReplaceTempView(s"${tableName}_tmp_view")
    sql(s"insert overwrite table ${resultTable} partition (ds=$ds) select * from ${tableName}_tmp_view")

    df.printSchema()
    df.show(100)
    println("company calc end! " + new Date().toString)

    spark.stop();
  }


  private def trans(code2Name: Broadcast[Map[String, String]], r: Row) = {

    val id = r.getAs[Long]("id")
    val company_name = r.getAs[String]("company_name")
    val reg_capital = r.getAs[String]("reg_capital")
    val actual_capital = r.getAs[String]("actual_capital")
    val category_code = r.getAs[String]("category_code")
    val company_org_type = r.getAs[String]("company_org_type")
    val reg_status = r.getAs[String]("reg_status")
    val estiblish_time = r.getAs[String]("estiblish_time")
    val to_time = r.getAs[String]("to_time")
    val reg_location = r.getAs[String]("reg_location")

    val r1 = DimScoreV2.bean2Map(DimScoreV2.registCapiScore(reg_capital))
    val r2 = DimScoreV2.bean2Map(DimScoreV2.recCapScore(actual_capital, reg_capital))
    val r3 = DimScoreV2.bean2Map(DimScoreV2.industryScore(code2Name.value.getOrElse(category_code, null)))
    val r4 = DimScoreV2.bean2Map(DimScoreV2.econKindScore(company_org_type, "")) //注意解决是否上市
    val r5 = DimScoreV2.bean2Map(DimScoreV2.eciStatusScore(reg_status))
    val r6 = DimScoreV2.bean2Map(DimScoreV2.startDateScore(estiblish_time))
    val r7 = DimScoreV2.bean2Map(DimScoreV2.termScore(to_time))
    val r8 = DimScoreV2.bean2Map(DimScoreV2.addressScore(reg_location))

    Seq(
      (id, company_name, r1.get("kind"), r1.get("project"), r1.get("type"), r1.get("score"), r1.get("total"), r1.get("extraScore")),
      (id, company_name, r2.get("kind"), r2.get("project"), r2.get("type"), r2.get("score"), r2.get("total"), r2.get("extraScore")),
      (id, company_name, r3.get("kind"), r3.get("project"), r3.get("type"), r3.get("score"), r3.get("total"), r3.get("extraScore")),
      (id, company_name, r4.get("kind"), r4.get("project"), r4.get("type"), r4.get("score"), r4.get("total"), r4.get("extraScore")),
      (id, company_name, r5.get("kind"), r5.get("project"), r5.get("type"), r5.get("score"), r5.get("total"), r5.get("extraScore")),
      (id, company_name, r6.get("kind"), r6.get("project"), r6.get("type"), r6.get("score"), r6.get("total"), r6.get("extraScore")),
      (id, company_name, r7.get("kind"), r7.get("project"), r7.get("type"), r7.get("score"), r7.get("total"), r7.get("extraScore")),
      (id, company_name, r8.get("kind"), r8.get("project"), r8.get("type"), r8.get("score"), r8.get("total"), r8.get("extraScore"))
    )
  }
}
